#ifdef _WIN32
    #include "windows.h"
#endif

#ifndef POSITION_CONTROLLER_H_
#define POSITION_CONTROLLER_H_

#ifdef __cplusplus
extern "C" {
#endif

// Let the robots pivot rotating for a while. Thanks to swistrack we know the diameter of the circle formed by the robot
// movement and knowing the real diameter of the robot we can automatically extract the transformation factor.
double computePixelToCmFactor();

// Open the file used to print debugging data.
void enableDebugToFile();

// Close the file used to print debugging data.
void disableDebugToFile();

// Print data to file in csv format. It is useful to debug the controller and implement new algorithms offline.
void printDebugToFile();

// Since the robot IR transmitter detected by the camera is not centered (actually it is on the back side of the robot), then we need to
// estimate the real center of the robot. Basically the current detected position of the IR is prolonged towards the orientation line by
// an amount equivalent to the robot radius (this is of course a rough estimation that should be valid only when the robot is moving straight).
void getCenterOfBasic(unsigned int robotIndex);

// This version of the function try to get a better estimation of the robot center position taking into account also the odometry information
// coming from the robot. It uses either odometry information or swistrack information (do not merge them) and discriminates between one and the
// other based on current rotation speed: if low rotation speed then uses swistrack (almost staright), otherwise takes odometry.
void getCenterOfAdvanced(unsigned int robotIndex);

// Return the angle (-180..180) respect to the target of the robot id passed as argument
void getRotationToTargetOf(unsigned int robotIndex);

// Return the distance (pixels) to the target of the robot id passed as argument
void getDistanceToTargetOf(unsigned int robotIndex);

// Tell which particle number corresponds to which robot and change the addresses sequence accordingly
int identifyRobotParticles();

// Execute the position controller that will lead the robot to the target point.
void positionControlExec();

// Initialize the controller.
void startPositionController(int *robotAddr, int numRobots);

// Stop the controller.
void stopPositionController();

// Print the information (such as position, orientation, target, ...) to a terminal window.
void printOnTerminal(int id);
void printAllOnTerminal();

#ifdef __cplusplus
}
#endif

#endif // POSITION_CONTROLLER_H_



